
# The author: haoyan zhang
#E-mail: haoy.zhang@foxmail.com



import numpy as np

class solve(object):
    def __init__(self):
        pass
    def sta_a(self,c):
    
        c.calc_deleted_KG_matrix()
    
        KG,Force = c.KG_keeped,c.Force_keeped
        c._Disp_keeped = np.linalg.solve(KG,Force)
    
    
        for i,val in enumerate(c.keeped):
            I = val%c.mndof
            J = int(val/c.mndof)
            c.nodes[J].disp[c.nAk[I]] = c.Disp_keeped[i]
    
                    
        for el in c.get_elements():
            el.evaluate()
    
        c._is_c_solved = True

    def kin_a(self,c):
        from scipy import linalg as sl
        if not c._is_inited:
            c.calc_KG()
        c.calc_deleted_KG_matrix()
        c.calc_MG()
        c.calc_deleted_MG_matrix()
        
        w1,self.model = sl.eig(c.KG_keeped,c.MG_keeped)
        self.w = np.sqrt(w1)
        T = 2*np.pi/self.w
        self.freq = 1/T
        
    def newmark_beta(self,c,omega,pxi1=0.02,pxi2=0.04,n1=256,delta=0.5,beta=0.25):

        from scipy import linalg as sl
        w1,model = sl.eigh(c.KG_keeped,c.MG_keeped)
        f = np.sqrt(w1)
        omega1= f[1]
        omega2= f[0]
        aa=2*omega1*omega2*(pxi1*omega2-pxi2*omega1)/(omega2**2-omega1**2)
        bb=2*(omega2*pxi2-omega1*pxi1)/(omega2**2-omega1**2)
        C=aa* c._MG_keeped+bb* c._KG_keeped
    
        dt=2*np.pi/(omega*n1)
        
        N=60*n1
        self.wy=np.zeros(( c._KG_keeped.shape[0],N+1))
        sd=np.zeros(( c._KG_keeped.shape[0],N+1))
        jsd=np.zeros(( c._KG_keeped.shape[0],N+1))
        a0=1/(beta*dt**2);
        a1=delta/(beta*dt)
        a2=1/(beta*dt)
        a3=1/(2*beta)-1
        a4=delta/beta-1
        a5=dt/2*(delta/beta-2)
        a6=dt*(1-delta)
        a7=delta*dt
        P1=a0* c._MG_keeped+a1* C+ c._KG_keeped
        for i in np.arange(0,N):
            t=i*dt
            Qe= c._Force_keeped*np.sin(omega*t)+np.dot( c._MG_keeped,(a0* self.wy[:,i-1]+a2* sd[:,i-1]+a3* jsd[:,i-1]))+np.dot( C,(a1* self.wy[:,i-1]+a4* sd[:,i-1]+a5* jsd[:,i-1]))
            self.wy[:,i]=np.dot(np.linalg.inv( P1), Qe)
            jsd[:,i]=a0*( self.wy[:,i]- self.wy[:,i-1])-a2* sd[:,i-1]-a3* jsd[:,i-1]
            sd[:,i]= sd[:,i-1]+a6* jsd[:,i-1]+a7* jsd[:,i]


    def nonl_fri(self,K,M,Omega,n,n_0,freq1=253.52,freq2=1582.96,rho=7850,b=0.06, h=0.007,L=0.15,R=0.35,y1=0.04,y2=0.04,Amp_ex=500,kjq=8e6,miu=0.2,alfa=0.25,beta=0.5):
        
        sdof=2*(1+n)-1
        omega_ex=n_0*Omega
        sunalpha=(1/3)*np.pi
        sunbeta=sunalpha
        A=b*h
        oumiga1=freq1*2*np.pi;oumiga2=freq2*2*np.pi;
        
        alafa=2*(y2/oumiga2-y1/oumiga1)/(1/(oumiga2**2)-1/(oumiga1**2));
        beita=2*(y2*oumiga2-y1*oumiga1)/(oumiga2**2-oumiga1**2);
        C=alafa*M+beita*K;
        
        n3=512;
        dt=2*np.pi/(omega_ex*n3);
        nt=600*n3;
        
        f_fric=np.random.rand(1)/1e30;f_fric_v=np.zeros((sdof,1));
        f_fric_v[0]=-2*f_fric;
        f0=np.zeros((sdof,1));
        d_former=np.random.rand(1)/1e30;w_former=np.random.rand(1)/1e30;
        
        N1=rho*A*Omega**2*(((R+L)**2-R**2)/2)
        N2=N1*(np.cos(sunbeta)/np.sin(sunalpha+sunbeta))
        
        x1=np.random.rand(sdof,1)/1e30
        v1=np.random.rand(sdof,1)/1e30
        
        aa=np.dot(np.linalg.inv(M),f0-np.dot(C,v1)-np.dot(K,x1))
        a0=1/(alfa*dt**2);
        a1=beta/(alfa*dt)
        a2=1/(alfa*dt)
        a3=1/(2*alfa)-1
        a4=beta/alfa-1
        a5=dt/2*(beta/alfa-2)
        a6=dt*(1-beta)
        a7=beta*dt
        ke=K+a0*M+a1*C
        
        self.xt=[]
        self.xo=[]
        self.fo=[]
        self.xd=[]
        for i in range(nt):
            
            t=(i+1)*dt
            fi=Amp_ex*np.cos(omega_ex*t)
            f0[29]=fi
    
            f2=f0+f_fric_v
            fe=f2+np.dot(M,a0*x1+a2*v1+a3*aa)+np.dot(C,a1*x1+a4*v1+a5*aa)
            x2=np.dot(np.linalg.inv(ke),fe)
            aa2=a0*(x2-x1)-a2*v1-a3*aa
            v2=v1+a6*aa+a7*aa2
            x1=x2
            v1=v2
            aa=aa2
            d_curent=x1[0]/np.sin(sunalpha)
            
            if kjq*(abs(d_curent-w_former)[0])<miu*N2:
                f_fric=kjq*(d_curent-w_former)
                w_curent=w_former
            elif kjq*(abs(d_curent-w_former)[0])>=miu*N2:
                f_fric=miu*N2*np.sign(d_curent-w_former)
                w_curent=d_curent-miu*N2/kjq*np.sign(f_fric)
            f_fric_v[0]=-2*f_fric*np.sin(sunalpha)
            d_former=d_curent
            w_former=w_curent
            
            if i+1>100*n3:
    
                self.xt.append(t)
                self.xo.append(x1[0])
                self.fo.append(f_fric)
                self.xd.append(v1[0])







# The author: haoyan zhang
#E-mail: haoy.zhang@foxmail.com


def no_conn(L,end = False):
    pool = tuple(L)
    n = len(pool)
    assert n >=2,"Length of iterable must greater than 3"
    if end is True:
        indices = list(range(n))
        for i in indices:
            if i < n -1:
                yield (pool[i],pool[i+1])
            else:
                yield (pool[i],pool[0])
    if end is False:
        indices = list(range(n-1))
        for i in indices:
            yield (pool[i],pool[i+1])
    
import matplotlib.pyplot as plt
import numpy as np

from matplotlib.font_manager import FontProperties
font = FontProperties(fname=r"c:\windows\fonts\simsun.ttc", size=15)

def Resp(nn,wy,n1=256):
    N=60*n1
    T=np.linspace(0,N+1,N+1)
    xX1=wy[2*nn-1,:]
    plt.figure(1)
    plt.plot(T,xX1,'b')
    plt.title('pyplot x1-t')
    plt.xlabel('time(s)')
    plt.ylabel('x1')
    return plt.figure(1)

def Vib_mode(model,n,L,ord_n):
    V1=model[:,2*n-ord_n]
    V1=V1[0:2*n:2]
    V_1=np.zeros(n+1)
    V_1[0]=0
    V_1[1:n]=V1[0:n-1]
    x=np.arange(0,n+1,1)
    plt.figure(2)
    plt.plot(x*(L/n)*1000,V_1/max(abs(V_1)))
    plt.xlim(0,1200)
    plt.title('振型图',fontproperties=font)
    return plt.figure(2)

def Spectrum(omega,nn,wy,n1=256):
    signal=wy[2*nn-1,9000:]
    dt=2*np.pi/(omega*n1)
    Fs=1/dt
    Num=len(signal)
    Num1=int(Num/2)
    X=signal-np.mean(signal)
    y=np.fft.fft(X,Num-1)
    y1=2*abs(y)/Num
    n2=np.arange(0,Num1,1)
    f=Fs*n2/Num
    plt.figure(3,figsize=(8,8)) 
    f_out=f[1:Num1]
    y1_out=y1[1:Num1]
    plt.plot(f_out,y1_out)

    plt.title('频谱图',fontproperties=font)
    return plt.figure(3)

def With_fri_resp(x,y):
    plt.figure(4)
    plt.plot(x,y)

def Frictional_resp(x,y):
    plt.figure(5)
    plt.plot(x,y)
    
def Fri_hysteresis_curve(x,y):
    plt.figure(6)
    plt.plot(x,y)


import setuptools

with open("README.md", "r", encoding="UTF-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="sfe",
    version="1.0.5",
    author="haoy_zhang",
    author_email="haoy.zhang@foxmail.com",
    description="Python and finite Elements",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitee.com/yonvone/sfe.git",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    extras_require={"sfe": ["python"]},
)




# The author: haoyan zhang
#E-mail: haoy.zhang@foxmail.com


from sfe.Node import *




class Element(object):

    def __init__(self,nodes):
        for nd in nodes:
            assert issubclass(type(nd),Node),"Must be Node type"
            
        self._nodes = nodes
        self._dim = 1
        self._etype = self.__class__.__name__
        self._numb = None
        self._eIk = None
        self._ndof = None 
        self._Ke = None
        self._Me = None
        self._volume = None
        self.init_keys()
        self.init_nodes(nodes)
        self.init_unknowns()
        self._force =  dict.fromkeys(self.eIk,0.)
        self.dens = None
        self.t = 1.
        
         
    def __repr__(self):
        return "%s Element: %r"%(self.etype,self.nodes,)
    
    @property
    def volume(self):
        return self._volume

    @property
    def B(self):
        return self._B

    @property
    def D(self):
        return self._D

    @property
    def dim(self):
        return self.nodes[0].dim
    
    @property
    def eIk(self):
        return self._eIk
    
    @property
    def Ke(self):
        return self._Ke

    @property
    def Me(self):
        return self._Me
    
    @property
    def ndof(self):
        return self._ndof
    
        
    @property
    def nodes(self):
        return self._nodes
    
    @property
    def etype(self):
        return self._etype
        
    @property
    def numb(self):
        return self._numb
    
    @property
    def non(self):
        return len(self._nodes)
    
    def set_eIk(self,val):
        self._eIk = val

    def get_eIk(self):
        return self._eIk

    def get_ndof(self):
        return self._ndof
    
    def get_element_type(self):
        return self._etype

    def get_nodes(self):
        return self._nodes

    def init_nodes(self,nodes):
        v = np.array(nodes[0].coord)-np.array(nodes[1].coord)
        le = np.linalg.norm(v)
        self._volume = le
        

    def init_keys(self):
        if self.dim == 2:
            self.set_eIk(("N","Ty","Mz"))
        if self.dim == 3:
            self.set_eIk(("N","Ty","Tz","Mx","My","Mz"))
        

    @property
    def T(self):
        return self._T

    @property
    def me(self):
        return self._me
    
    @property
    def ke(self):
        return self._ke

    @property
    def force(self):
        return self._force

    def calc_Ke(self):
        self.calc_T()
        self.calc_ke()
        self._Ke = self.t*np.dot(np.dot(self.T.T,self.ke),self.T)

    def calc_Me(self):
        self.calc_T()
        self.calc_me()
        self._Me = np.dot(np.dot(self.T.T,self.me),self.T)
        
    def evaluate(self):
        u = np.array([[nd.disp[key] for nd in self.nodes for key in nd.nAk[:self.ndof]]])
        self._undealed_force = np.dot(self.T,np.dot(self.Ke,u.T))
        self.distribute_force()

    def distribute_force(self):
        n = len(self.eIk)
        for i,val in enumerate(self.eIk):
            self._force[val] += self._undealed_force[i::n]



# The author: haoyan zhang
#E-mail: haoy.zhang@foxmail.com







import numpy as np

class Node(object): 
    
    def __init__(self,*coord):
        self.init_coord(*coord) 
        self._numb = None 
        self._nAk = None 
        self._nBk = None 
        self.init_keys() 
        self._dof = len(self.nAk) 
        self._disp = dict.fromkeys(self.nAk,0.) 
        self._force = dict.fromkeys(self.nBk,0.) 
     
    def __repr__(self):  
        return "Node:%r"%(self.coord)
    
    def init_coord(self,*coord): 
        self._x = 0
        self._y = 0
        self._z = 0
        
        if len(coord) == 1:
            self.dim = len(coord[0])
            if self.dim == 2:
                self._x = coord[0][0]*1.
                self._y = coord[0][1]*1.
                self.coord = (self.x,self.y)
            elif self.dim == 3:
                self._x = coord[0][0]*1.
                self._y = coord[0][1]*1.
                self._z = coord[0][2]*1.
                self.coord = (self.x,self.y,self.z)
            else:
                raise AttributeError("Node dimension is 2 or 3")
            
        elif len(coord) == 2:
            self.coord = tuple(coord)
            self.dim = 2
            self._x = coord[0]*1.
            self._y = coord[1]*1.
            self.coord = (self.x,self.y)
        elif len(coord) == 3:
            self.coord = tuple(coord)
            self.dim = 3
            self._x = coord[0]*1.
            self._y = coord[1]*1.
            self._z = coord[2]*1.
            self.coord = (self.x,self.y,self.z)
        else:
            raise AttributeError("Node dimension is 2 or 3")


    @property
    def nBk(self):
        return self._nBk

    @property
    def nAk(self):
        return self._nAk


    def set_nAk(self,val):
        self._nAk = val

    def get_nAk(self):
        return self._nAk

    def set_nBk(self,val):
        self._nBk = val

    def get_nBk(self):
        return self._nBk
    
    @property
    def x(self):
        return self._x
    
    @property
    def y(self):
        return self._y

    @property
    def z(self):
        return self._z

    @property
    def numb(self):
        return self._numb

    @property
    def force(self):
        return self._force
        
    @property
    def disp(self):
        return self._disp
        
    def init_keys(self):
        if self.dim == 2:
            self.set_nAk(("Ux","Uy","Phz"))
            self.set_nBk(("Fx","Fy","Mz"))
        elif self.dim == 3:
            self.set_nAk(("Ux","Uy","Uz","Phx","Phy","Phz"))
            self.set_nBk(("Fx","Fy","Fz","Mx","My","Mz"))

    def init_unknowns(self,*unknowns):
        for key in unknowns:
            if key in self.nAk:
                self._disp[key] = None
            else:
                raise AttributeError("Unknow disp name(%r)"%(unknowns,))
    
    def set_force(self,**forces):
        for key in forces.keys():
            if key in self.nBk:
                self._force[key] += forces[key]
            else:
                raise AttributeError("Unknow focre name(%r)"%(forces,))
    

    def get_force(self):
        return self._force

    def set_disp(self,**disp):
        for key in disp.keys():
            if key in self.nAk:
                self._disp[key] = disp[key]
            else:
                raise AttributeError("Unknow disp name(%r)"%(disp,))
              
    def get_disp(self):
        return self._disp